package com.vandyo.sentry.core.cases;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.vandyo.sentry.core.dto.ErrStatus;
import com.vandyo.sentry.core.dto.Res;
import com.vandyo.sentry.core.exceptions.CaseException;
import com.vandyo.sentry.core.tools.Signature;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 车体检案例
 */
public class CarDetectDecorator extends CaseDecorator {

    private final static String carDetectUrl = "/car/car_detect";

    public CarDetectDecorator(Case aCase) {
        super(aCase);
    }

    /**
     * FIXME: 需要优化相同的代码有些请求 只需要修改一个url
     * @return
     */
    @Override
    public Res<Map<String, String>> check() throws CaseException {
        Res<Map<String,String>> oldRes = super.check();
        if (oldRes.getSuccess()) {
            Map<String, String> carMap = oldRes.getData();
            Res<Map<String,String>> newRes = new Res<>();
            newRes.setSId(oldRes.getSId());
            newRes.setUId(oldRes.getUId());

            if (Objects.isNull(carMap) || !carMap.containsKey("cid")){
                newRes.setStatus(ErrStatus.ErrUnexpected);
                newRes.setSuccess(false);
            }else {
                String cid = carMap.get("cid");
                Map<String,String> paramMap = new HashMap<>();
                paramMap.put("cid",cid);
                paramMap.put("uid",oldRes.getUId());
                try {
                    HttpResponse response = HttpRequest.get(
                            Signature.host + carDetectUrl + "?"
                                    + Signature.getUrlParamsByMap(Signature.sign(paramMap, oldRes.getSId()))
                    ).execute();

                    if (response.isOk()) {
                        JSONObject jsonObject = JSONUtil.parseObj(response.body());
                        //todo 检测 返回值可用性
                        newRes.setData(carMap);
                        newRes.setSuccess(true);
                    } else {
                        newRes.setKey(carDetectUrl);
                        newRes.setStatus(Signature.matchStatus(response.getStatus()));
                        newRes.setSuccess(false);
                    }
                }catch (Exception e){
                    if (e.getMessage().toLowerCase().contains("timeout")){
                        throw new CaseException(carDetectUrl, ErrStatus.ErrTimeout);
                    }else {
                        throw new CaseException(carDetectUrl,ErrStatus.ErrUnknown);
                    }
                }
            }
            return newRes;
        }else {
            oldRes.setIgnore(true);
            return oldRes;
        }
    }
}
